@@ -1,19 +1,66 @@
 import unittest
 import sys
 import functools
+import shutil
 
 sys.path.insert(0, '../reticulator')
 from reticulator import *
 
 def get_packs():
     project = Project('./content/bp/', './content/rp/')
+    project.resource_pack.output_path = './out/rp/'
+    project.behavior_pack.output_path = './out/bp/'
     return project.behavior_pack, project.resource_pack
 
-def create_output_directory():
-    os.mkdir('test_output')
+def get_saved_packs():
+    project = Project('./out/bp/', './out/rp/')
+    return project.behavior_pack, project.resource_pack
 
-def clean_up_output_directory():
-    os.remove('test_output')
+def prepare_output_directory():
+    try:
+        shutil.rmtree('./out')
+    except OSError as e:
+        pass
+
+    os.mkdir('./out')
+
+class TestRenderControllers(unittest.TestCase):
+    def setUp(self) -> None:
+        self.bp, self.rp = get_packs()
+
+    def test_render_controller_file(self):
+        self.assertEqual(len(self.rp.render_controller_files), 1)
+
+    def test_render_controller(self):
+        self.assertEqual(len(self.rp.render_controllers), 2)
+
+    def test_get_render_controller(self):
+        self.rp.get_render_controller('controller.render.dolphin')
+
+    def test_get_render_controller_file(self):
+        self.rp.get_render_controller_file('render_controllers/dolphin.render_controller.json')
+    
+    def test_internals(self):
+        self.assertEqual(self.rp.get_render_controller('controller.render.dolphin').file.format_version, '1.8.0')
+
+class TestLootTables(unittest.TestCase):
+    def setUp(self) -> None:
+        self.bp, self.rp = get_packs()
+
+    def test_loot_table(self):
+        self.assertEqual(len(self.bp.loot_tables), 2)
+        self.assertEqual(self.bp.loot_tables[0].file_name, 'dolphin.json')
+    
+    def test_pools(self):
+        self.assertEqual(len(self.bp.loot_tables[0].pools), 1)
+
+    def test_loot_from_entity(self):
+        dolphin = self.bp.get_entity('minecraft:dolphin')
+        group = dolphin.get_component_group('dolphin_adult')
+        component = group.get_component('minecraft:loot')
+        table_name = component.data['table']
+        loot_table = self.bp.get_loot_table(table_name)
+        self.assertEqual(len(loot_table.pools), 1)
 
 class TestSounds(unittest.TestCase):
     def setUp(self) -> None:
@@ -71,12 +118,12 @@ class TestFunctions(unittest.TestCase):
     
     def test_getting_function(self):
         # Getting function by path can take multiple path formats
-        self.assertTrue(self.bp.get_function('kill_all_safe.mcfunction'))
-        self.assertTrue(self.bp.get_function('teleport/home.mcfunction'))
+        self.assertTrue(self.bp.get_function('functions/kill_all_safe.mcfunction'))
+        self.assertTrue(self.bp.get_function('functions/teleport/home.mcfunction'))
 
     def test_non_existent_function(self):
         with self.assertRaises(AssetNotFoundError):
-            self.bp.get_function('no_function.mcfunction')
+            self.bp.get_function('functions/no_function.mcfunction')
 
     def test_editing_command(self):
         function = self.bp.functions[0]
@@ -110,7 +157,7 @@ class TestDirty(unittest.TestCase):
     def setUp(self) -> None:
         self.bp, self.rp = get_packs()
         self.entity = self.bp.get_entity('minecraft:dolphin')
-        self.function = self.bp.get_function('kill_all_safe.mcfunction')
+        self.function = self.bp.get_function('functions/kill_all_safe.mcfunction')
         self.command = self.function.commands[0]
         self.component = self.entity.get_component('minecraft:type_family')
 
@@ -200,7 +247,7 @@ class TestParticle(unittest.TestCase):
         self.assertEqual(component.get_jsonpath('num_particles'), 20)
         self.assertEqual(component.data['num_particles'], 20)
 
-class TestShortnameResourceTriple(unittest.TestCase):
+class TestAnimationTriple(unittest.TestCase):
     def setUp(self) -> None:
         self.bp, self.rp = get_packs()
         self.dolphin = self.rp.get_entity('minecraft:dolphin')
@@ -217,7 +264,7 @@ class TestShortnameResourceTriple(unittest.TestCase):
         animation = animations[0]
         self.assertEqual(animation.shortname, 'move')
         self.assertEqual(animation.resource.id, 'animation.dolphin.move')
-        self.assertEqual(animation.id, 'animation.dolphin.move')
+        self.assertEqual(animation.identifier, 'animation.dolphin.move')
         self.assertEqual(animation.exists(), True)
 
     def test_missing_resources(self):
@@ -232,10 +279,47 @@ class TestShortnameResourceTriple(unittest.TestCase):
         animation = animations[-1]
 
         self.assertEqual(animation.shortname, 'missing')
-        self.assertEqual(animation.id, 'animation.guardian.missing')
-        self.assertEqual(animation.resource, None)
+        self.assertEqual(animation.identifier, 'animation.guardian.missing')
         self.assertEqual(animation.exists(), False)
 
+        with self.assertRaises(AssetNotFoundError):
+            animation.resource
+
+    def test_saving(self):
+        """
+        Tests that we can save AnimationTriple
+        """
+
+        # Edit the resource
+        animation : AnimationTriple = self.dolphin.animations[0]
+        animation.shortname = 'new_shortname'
+        animation.identifier = 'new_identifier'
+
+        # Save the resource
+        prepare_output_directory()
+        self.rp.save()
+
+        # Test the saved resource
+        out_bp, out_rp = get_saved_packs()
+        animation = out_rp.get_entity('minecraft:dolphin').animations[0]
+
+        self.assertEqual(animation.shortname, 'new_shortname')
+        self.assertEqual(animation.identifier, 'new_identifier')
+
+        # Raise error for the renamed resource
+        with self.assertRaises(AssetNotFoundError):
+            self.dolphin.get_animation('move')
+
+class TestEntityFileBP(unittest.TestCase):
+    def setUp(self) -> None:
+        self.bp, self.rp = get_packs()
+        self.entity = self.bp.get_entity('minecraft:dolphin')
+
+    def test_component_groups(self):
+        self.assertEqual(len(self.entity.component_groups), 7)
+        group = self.entity.get_component_group('dolphin_adult')
+        self.assertEqual(group.id, 'dolphin_adult')
+        self.assertEqual(len(group.components), 4)
 
 class TestEntityFileRP(unittest.TestCase):
     def setUp(self) -> None:
